#!/usr/bin/perl

use strict;
use padlock;

#AES
package AES_Class;
sub new{
	my $class = shift();
	my $self = {};
	$self->{"key"} = undef;
	$self->{"key_length"} = undef;
	$self->{"mode"} = undef;
	$self->{"iv"} = undef;
	bless $self,$class;
	return $self;
}

sub perl_ace_available{
	my ($self)=@_;
	my $flag = padlock->perl_padlock_ace_available();
};

sub perl_ace2_available{
	my ($self)=@_;
	my $flag = padlock->perl_padlock_ace2_available();
};

sub perl_aes_begin{
	my ($self) = @_;
	$self->{"key"} = "00000000000000000000000000000000";
	$self->{"key_length"} = 128;
	$self->{"mode"} = "CBC";
	$self->{"iv"} = "0000000000000000";
};

sub perl_aes_setkey{
	my ($self,$key,$keylen) = @_;
	$self->{"key"} = padlock->perl_aes_setkey($key, $keylen);
	$self->{"key_length"}=$keylen;
	return $self->{"key"};
};

sub perl_aes_setiv{
	my ($self,$mode,$iv) = @_;
	$self->{"iv"} = padlock->perl_aes_setiv($mode, $iv);
	$self->{"mode"}=$mode;
	return $self->{"iv"};
};

sub perl_aes_encrypt{
	my ($self, $text) = @_;
	my $dst = padlock->perl_padlock_aes_encrypt($self->{"key"}, $self->{"key_length"}, $self->{"mode"}, $self->{"iv"}, $text);
	return $dst;
};

sub perl_aes_decrypt{
	my ($self, $text) = @_;
	my $dst = padlock->perl_padlock_aes_decrypt($self->{"key"}, $self->{"key_length"}, $self->{"mode"}, $self->{"iv"}, $text);
	return $dst;
};

sub perl_aes_close{
	my ($self, $ptr) = @_;
	$self->{"mode"} = undef;
	$self->{"key_length"} = undef;
	$self->{"mode"} = undef;
	$self->{"iv"} = undef;
};

#RNG
package RNG_Class;
sub new{
	my $class = shift();
	my $self = {};
	bless $self, $class;
	return $self;
};

sub perl_rng_available{
	my ($self) = @_;
	my $flag = padlock->perl_padlock_rng_available();
};

sub perl_rng_rand{
	my ($self, $randlen) = @_;
	my $dst = padlock->perl_padlock_rng_rand($randlen);
	return $dst;
};

#PMM
package PMM_Class;
sub new{
	my $class = shift();
	my $self = {};
	bless $self,$class;
	return $self;
};

sub perl_pmm_available{
	my ($self)=@_;
	my $flag = padlock->perl_padlock_pmm_available();
};

sub perl_pmm2_available{
	my ($self)=@_;
	my $flag = padlock->perl_padlock_pmm2_available();
};

sub perl_pmm{
	my ($self, $a, $b, $m, $len) = @_;
	unless($len =~/^\d+$/){
		die "pmm len input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_padlock_pmm($a, $b, $m, $len);
	return $dst;
};

#SHA1
package SHA1_Class;
sub new{
	my $class  =  shift();
	my $self = {};
	$self->{"hash"} = shift();
	$self->{"key_length"} = shift();
	bless $self,$class;
	return $self;
}

sub perl_phe_available{
	my ($self) = @_;
	my $flag = padlock->perl_padlock_phe_available();
};

sub perl_sha1{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_sha1 input is wrong or the memory allocation error\n";
	}

        unless($length =~/^\d+$/){
		die "perl_sha1 input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_sha1($src,$length);
	if($dst eq "wronginput"){
		die "perl_sha1 input is wrong or the memory allocation error\n";
	}
	return $dst;	
};

sub perl_phe_partialavailable{
	my ($self) = @_;
	my $flag = padlock->perl_phe_partial_available();
};

sub perl_phe_partial_sha1_init{
	my ($self) = @_;
	$self->{"hash"} = "67452301efcdab8998badcfe10325476c3d2e1f0";
	$self->{"key_length"} = 0;
};

sub perl_phe_partial_sha1_update{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_phe_partial_sha1_update input is wrong or the memory allocation error\n";
	}

        unless($length =~/^\d+$/){
		die "perl_phe_partial_sha1_update input is wrong or the memory allocation error\n";
        }
	$self->{"hash"} = padlock->perl_phe_partial_sha1_update($src,$self->{"hash"},$length);
	$self->{"key_length"} = $self->{"key_length"} + $length;
	return $self->{"hash"};
};

sub perl_phe_partial_sha1_final{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_phe_partial_sha1_final input is wrong or the memory allocation error\n";
	}
        unless($length =~ /^\d+$/){
		die "perl_phe_partial_sha1_final input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_partial_sha1_final($src,$self->{"hash"},$self->{"key_length"},$length);	
	return $dst;
};

#SHA256
package SHA256_Class;
sub new{
	my $class  =  shift();
	my $self = {};
	$self->{"hash"} = shift();
	$self->{"key_length"} = shift();
	bless $self,$class;
	return $self;
}

sub perl_phe_available{
	my ($self) = @_;
	my $flag = padlock->perl_padlock_phe_available();
};

sub perl_sha256{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_sha256 input is wrong or the memory allocation error\n";
	}
        unless($length =~ /^\d+$/){
		die "perl_sha256 input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_sha256($src,$length);
	print "$dst\n";
	if($dst eq "wronginput"){
		die "perl_sha256 input is wrong or the memory allocation error\n";
	}
	return $dst;
};

sub perl_phe_partialavailable{
	my ($self) = @_;
	my $flag = padlock->perl_phe_partial_available();
};

sub perl_phe_partial_sha256_init{
	my ($self) = @_;
	$self->{"hash"} = "6a09e667bb67ae853c6ef372a54ff53a510e527f9b05688c1f83d9ab5be0cd19";
	$self->{"key_length"} = 0;
};

sub perl_phe_partial_sha256_update{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_phe_partial_sha1_update is wrong or the memory allocation error\n";
	}
        unless($length =~/^\d+$/){
		die "perl_phe_partial_sha256_update input is wrong or the memory allocation error\n";
        }
	$self->{"hash"} = padlock->perl_phe_partial_sha256_update($src,$self->{"hash"},$length);
	$self->{"key_length"} = $self->{"key_length"} + $length;
	return $self->{"hash"};
};

sub perl_phe_partial_sha256_final{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_phe_partial_sha256_final input is wrong or the memory allocation error\n";
	}
        unless($length =~/^\d+$/){
		die "perl_phe_partial_sha256_final input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_partial_sha256_final($src,$self->{"hash"},$self->{"key_length"},$length);	
	return $dst;
};

#SHA384
package SHA384_Class;
sub new{
	my $class  =  shift();
	my $self = {};
	$self->{"hash"} = shift();
	$self->{"key_length"} = shift();
	bless $self,$class;
	return $self;
}

sub perl_phe_available{
	my ($self) = @_;
	my $flag = padlock->perl_padlock_phe_available();
};

sub perl_sha384{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_sha384 input is wrong or the memory allocation error\n";
	}

        unless($length =~/^\d+$/){
		die "perl_sha384 input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_sha384($src,$length);
	if($dst eq "wronginput"){
		die "perl_sha384 input is wrong or the memory allocation error\n";
	}
	return $dst;	
};

#SHA512
package SHA512_Class;
sub new{
	my $class  =  shift();
	my $self = {};
	$self->{"hash"} = shift();
	$self->{"key_length"} = shift();
	bless $self,$class;
	return $self;
}

sub perl_phe_available{
	my ($self) = @_;
	my $flag = padlock->perl_padlock_phe_available();
};

sub perl_sha512{
	my ($self,$src,$length) = @_;
	if(length($src) == 0 || $length <= 0 || int($length) != $length || length($src) != $length ){
		die "perl_sha512 input is wrong or the memory allocation error\n";
	}

        unless($length =~/^\d+$/){
		die "perl_sha512 input is wrong or the memory allocation error\n";
        }
	my $dst = padlock->perl_phe_sha512($src,$length);
	if($dst eq "wronginput"){
		die "perl_sha512 input is wrong or the memory allocation error\n";
	}
	return $dst;	
};

1
